/**
 * 定义常用界面组件对象
 * 依赖Jquery
 * 李志钢 2014-09-28
 * 2014-10-05 增加默认处理，给所有携带onclick的对象绑定为touch模式
 */

/*
 * 实现对事件的封装，适应模拟器+设备
 */
var PowerEvent = function() {

};

PowerEvent.$support = {
    transform3d : ('WebKitCSSMatrix' in window),
    touch : ('ontouchstart' in window)
};
PowerEvent.$E = {
    start : PowerEvent.$support.touch ? 'touchstart' : 'mousedown',
    move : PowerEvent.$support.touch ? 'touchmove' : 'mousemove',
    end : PowerEvent.$support.touch ? 'touchend' : 'mouseup',
    cancel : PowerEvent.$support.touch ? 'touchcancel' : '',
    transEnd : 'webkitTransitionEnd'
};

/**
 * PowerTouch
 * 绑定点击对象，实现对onclick的加速，同时实现点击的样式改变
 * $(selector).PowerTouch(options);
 */
(function($) {
    $.fn.PowerTouch = function(options) {
        var defaultOpts = {
            css : "touchState"
        };
        var opts = $.extend({}, defaultOpts, options);
        return this.each(function() {
            $this = $(this);
            var powerTouch = new PowerTouch($this);
            powerTouch.setCss(opts.css);
            $this.bind(PowerEvent.$E.start, function() {
                powerTouch.onTouch();
            });
            $this.bind(PowerEvent.$E.move, function() {
                powerTouch.onMove();
            });
            $this.bind(PowerEvent.$E.end, function() {
                powerTouch.onEnd();
            });
        });
    };
})(jQuery);

var PowerTouch = function($obj) {
    var me = this;
    me.target = $obj;
    this.clickFunc = $obj.attr("onclick");
    $obj.removeAttr("onclick");
};

PowerTouch.prototype = {
    setCss : function(css) {
        this.mCss = css;
    },
    onTouch : function() {
        //记录点击的位置
        this.pointX = this.getPage(event, 'pageX');
        this.pointY = this.getPage(event, 'pageY');
        this.bClick = true;
        //添加css
        if ( typeof (this.mCss) != "undefined" && this.mCss != "") {
            var c = this.mCss;
            var t = this.target;
            t.addClass(c);
        }
    },
    onMove : function() {
        if( !this.bClick )
            return;
        //如果移动了一定的位置，就取消点击行为
        var pointX = this.getPage(event, 'pageX');
        var pointY = this.getPage(event, 'pageY');
        if (Math.abs(pointX - this.pointX) > 15 || Math.abs(pointY - this.pointY) > 15 ) {
            this.bClick = false;
        }
        //一旦移动，就取消变色状态，因为后面的onEnd将可能不会被执行了
        var c = this.mCss;
        var t = this.target;
        t.removeClass(c);        
    },
    onEnd : function() {
        //去掉css
        if ( typeof (this.mCss) != "undefined" && this.mCss != "") {
            var c = this.mCss;
            var t = this.target;
            setTimeout(function() {
                t.removeClass(c);
            }, 1);
        }
        //触发onclick
        if (this.bClick) {
            if ( typeof (this.clickFunc) != 'undefined') {
                var f = this.clickFunc;
                setTimeout(function() {
                    eval(f);
                }, 10);
            }
        }
    },
    getPage : function(event, page) {
        return PowerEvent.$support.touch ? event.changedTouches[0][page] : event[page];
    }
};

/**
 * 边菜单组件
 * mainDiv  选择器，指向主div
 * menuDiv  选择器，指向菜单div 排版时，通过margin: -widthem来标识
 * side     菜单往哪边收缩  "left" "right"
 */
var PowerSideMenu = function(mainDiv, menuDiv, side) {
    var self = this;
    var state = "0";
    // 0 or 1

    this.getPage = function(event, page) {
        return PowerEvent.$support.touch ? event.touches[0][page] : event[page];
    };
    this.getTranslateX = function(x) {
        return PowerEvent.$support.transform3d ? 'translate3d(' + x + 'em, 0, 0)' : 'translate(' + x + 'em, 0)';
    };

    this.onStart = function(event, e1, e2) {
        self.touched = true;
        self.startPageX = self.getPage(event, 'pageX');
        self.currentX = 0;
    };
    this.onMove = function(event) {
        if (self.touched == true) {
            event.preventDefault();
            var pageX = self.getPage(event, 'pageX');
            var distX = pageX - self.startPageX;
            if (distX > 50) {
                self.state = '1';
                self.touched = false;
                $(mainDiv)[0].style.webkitTransitionDuration = '300ms';
                $(mainDiv)[0].style.webkitTransform = self.getTranslateX(10);
                $(menuDiv)[0].style.webkitTransitionDuration = '300ms';
                $(menuDiv)[0].style.webkitTransform = self.getTranslateX(10);
            }
        }
    };
    this.onEnd = function(event) {
        if (self.touched) {
            if (self.state == '1') {
                $(mainDiv)[0].style.webkitTransform = self.getTranslateX(0);
                $(menuDiv)[0].style.webkitTransform = self.getTranslateX(0);
                self.state = '0';
            }
        }
        self.touched = false;
    };
    $(mainDiv)[0].addEventListener(PowerEvent.$E.start, this.onStart, false);
    $(mainDiv)[0].addEventListener(PowerEvent.$E.move, this.onMove, false);
    $(mainDiv)[0].addEventListener(PowerEvent.$E.end, this.onEnd, false);
}; 


//-------------------slider-------------------
    function getTranslateX(x) {
        return PowerEvent.$support.transform3d ? 'translate3d('+x+'px, 0, 0)' : 'translate('+x+'px, 0)';
    }
    function getTranslateY(y) {
        return PowerEvent.$support.transform3d ? 'translate3d(0,'+y+'px, 0)' : 'translate(0,'+y+'px)';
    }
    function getPage (event, page) {
        return PowerEvent.$support.touch ? event.changedTouches[0][page] : event[page];
    }
    var zySlide = function(selector,dir,endFun,lock,transEnd){
        var self = this;
        self._locked= lock;
        self.endFun = endFun || null;
        self.transEnd = transEnd||null;
        self.nodes = [];
        //V:vertical
        //H:horizontal
        self.dir = dir||"H";
        if (selector.nodeType && selector.nodeType == 1) {
            self.element = selector;
        } else if (typeof selector == 'string') {
            self.id=selector;
            self.element = document.getElementById(selector) || document.querySelector(selector);
        }
        self._vendor=(window.navigator.userAgent.indexOf("Android 4.")>=0);
        
        if(self.dir=="H")
            self.element.style.webkitTransform = getTranslateX(0);
        else
            self.element.style.webkitTransform = getTranslateY(0);
        
        self.conf = {};
        self.touchEnabled = true;
        self.currentPoint = 0;
        self.currentXY = 0;

        self.refresh();
        
        //
        self.element.parentNode.addEventListener(PowerEvent.$E.start, self, false);
        self.element.parentNode.addEventListener(PowerEvent.$E.move, self, false);
        self.element.addEventListener(PowerEvent.$E.transEnd, self, false);
        if(PowerEvent.$E.cancel!="")
        {
            self.element.addEventListener(PowerEvent.$E.cancel, self, false);
            document.addEventListener(PowerEvent.$E.cancel, self, false);
        }
        document.addEventListener(PowerEvent.$E.end, self, false);

        return self;
    };

    zySlide.prototype = {
        handleEvent: function(event) {
            var self = this;

            switch (event.type) {
                case PowerEvent.$E.start:
                    self._touchStart(event);
                    break;
                case PowerEvent.$E.move:
                    self._touchMove(event);
                    break;
                case PowerEvent.$E.end:
                case PowerEvent.$E.cancel:
                    self._touchEnd(event);
                    break;
                case 'click':
                    self._click(event);
                    break;
                case PowerEvent.$E.transEnd:
                    if(self.transEnd)
                        self.transEnd(event);
                    break;
            }
        },
        addSection : function(obj){
            var self = this;
            self.element.appendChild(obj);
            self.refresh();
        },
        getSection : function(i){
            var self = this;
            var obj = self.nodes[i];
            if(obj.childNodes[1] && obj.childNodes[1].childNodes[1])
                return obj.childNodes[1].childNodes[1].childNodes[1];
            else
                return obj;
        },
        refresh: function() {
            var self = this;

            var conf = self.conf;

            // setting max point
            self.maxPoint = conf.point || (function() {
                var childNodes = self.element.childNodes,
                    itemLength = 0,
                    i = 0,
                    len = childNodes.length,
                    node;
                for(; i < len; i++) {
                    node = childNodes[i];
                    if (node.nodeType === 1) {
                        self.nodes[itemLength]=node;
                        itemLength++;
                    }
                }
                if (itemLength > 0) {
                    itemLength--;
                }
    
                return itemLength;
            })();
            
            // setting distance
            if(self.dir=="H")
                self.distance = parseInt(conf.distance || window.getComputedStyle(self.element,null).width);
            else
                self.distance = parseInt(conf.distance || window.getComputedStyle(self.element,null).height);
            // setting maxX maxY
            self.maxXY = conf.maxXY ? - conf.maxXY : - self.distance * self.maxPoint;
    
            self.moveToPoint(self.currentPoint);
        },
        hasNext: function() {
            var self = this;
    
            return self.currentPoint < self.maxPoint;
        },
        hasPrev: function() {
            var self = this;
    
            return self.currentPoint > 0;
        },
        toNext: function() {
            var self = this;

            if (!self.hasNext()) {
                return;
            }

            self.moveToPoint(self.currentPoint + 1);
        },
        toPrev: function() {
            var self = this;

            if (!self.hasPrev()) {
                return;
            }

            self.moveToPoint(self.currentPoint - 1);
        },
        moveToPoint: function(point) {
            var self = this;

            self.currentPoint = 
                (point < 0) ? 0 :
                (point > self.maxPoint) ? self.maxPoint :
                parseInt(point);
            
            self.element.style.webkitTransitionDuration = '500ms';
            self._setXY(- self.currentPoint * self.distance);
            

            var ev = document.createEvent('Event');
            ev.initEvent('css3flip.moveend', true, false);
            self.element.dispatchEvent(ev);
        },
        _setXY: function(xy) {
            var self = this;

            self.currentXY = xy;
            if(self.dir=="H")
                self.element.style.webkitTransform = getTranslateX(xy);
            else
                self.element.style.webkitTransform = getTranslateY(xy);
        },
        _touchStart: function(event) {
            var self = this;

            if(self._locked)
            {
                event.preventDefault();
                event.stopPropagation();
            }
            
            if (!self.touchEnabled) {
                return;
            }

            if (!PowerEvent.$support.touch) {
                event.preventDefault();
            }

            self.element.style.webkitTransitionDuration = '0';
            self.scrolling = true;
            self.moveReady = false;
            self.startPageX = getPage(event, 'pageX');
            self.startPageY = getPage(event, 'pageY');
            if(self.dir=="H")
                self.basePage = self.startPageX;
            else
                self.basePage = self.startPageY
            self.direction = 0;
            self.startTime = event.timeStamp;
        },
        _touchMove: function(event) {
            var self = this;
            
            if(self._locked)
            {
                event.preventDefault();
                event.stopPropagation();
            }
            
            if (!self.scrolling) {
                return;
            }
            var pageX = getPage(event, 'pageX'),
                pageY = getPage(event, 'pageY'),
                distXY,
                newXY,
                deltaX,
                deltaY;

            
            if (!self.moveReady) {
                deltaX = Math.abs(pageX - self.startPageX);
                deltaY = Math.abs(pageY - self.startPageY);
                if(self.dir=="H"){
                    if (deltaX>deltaY && deltaX > 5) {
                        if(self._vendor)//Android 4.0 need prevent default
                        {
                            event.preventDefault();
                        }
                        self.moveReady = true;
                        self.element.parentNode.addEventListener('click', self, true);
                    }
                    else if (deltaY > 5) {
                        self.scrolling = false;
                    }
                }else{
                    if (deltaY>deltaX && deltaY> 5) {
                        if(self._vendor)//Android 4.0 need prevent default
                        {
                            event.preventDefault();
                        }
                        self.moveReady = true;
                        self.element.parentNode.addEventListener('click', self, true);
                    }
                    else if (deltaX > 5) {
                        self.scrolling = false;
                    }
                }
            }
            if (self.moveReady) {
                //event.preventDefault();
                //event.stopPropagation();
                if(self.dir=="H")
                    distXY = pageX - self.basePage;
                else
                    distXY = pageY - self.basePage;
                newXY = self.currentXY + distXY;
                if (newXY >= 0 || newXY < self.maxXY) {
                    newXY = Math.round(self.currentXY + distXY / 3);
                }
                self._setXY(newXY);

                self.direction = distXY > 0 ? -1 : 1;
            }
            if(self.dir=="H")   
                self.basePage = pageX;
            else
                self.basePage = pageY;
        },
        _touchEnd: function(event) {
            var self = this;

            if(self._locked)
            {
                event.preventDefault();
                event.stopPropagation();
            }
            
            if (!self.scrolling) {

                return;
            }

            self.scrolling = false;

            var newPoint = -self.currentXY / self.distance;
            newPoint =
                (self.direction > 0) ? Math.ceil(newPoint) :
                (self.direction < 0) ? Math.floor(newPoint) :
                Math.round(newPoint);

            self.moveToPoint(newPoint);

            setTimeout(function() {
                self.element.parentNode.removeEventListener('click', self, true);
            }, 200);
            if(self.endFun)
                self.endFun();
        },
        _click: function(event) {
            var self = this;

            event.stopPropagation();
            event.preventDefault();
        },
        destroy: function() {
            var self = this;

            self.element.parentNode.removeEventListener(PowerEvent.$E.start, self);
            self.element.parentNode.removeEventListener(PowerEvent.$E.move, self);
            self.element.parentNode.removeEventListener(PowerEvent.$E.transEnd, self);
            self.element.parentNode.removeEventListener(PowerEvent.$E.cancel, self);
            document.removeEventListener(PowerEvent.$E.end, self);
            document.removeEventListener(PowerEvent.$E.cancel, self);
        }
    };


